@@ -1,12 +1,15 @@
-LM.bpm <- function(formula, data = list(), weights = NULL, subset = NULL, 
-                   Model, hess = TRUE, infl.fac = 1){
+LM.bpm <- function(formula, data = list(), weights = NULL, subset = NULL, Model, hess = TRUE){
 
   sp <- qu.mag <- y1.y2 <- y1.cy2 <- cy1.y2 <- cy1.cy2 <- cy <- cy1 <- NULL  
-  end <- 0
-  BivD <- "N"; PL <- "P"
+  
+  end <- data <- var <- G <- var.eig <- params <- resf <- VC <- respvec <- qu.mag <- X1 <- X2 <- gam1 <- gam2 <- 0
+  
+  BivD <- "N"
   fp <- FALSE
   
   if(!(Model %in% c("B", "BSS")) || missing(Model)) stop("Error in parameter Model value. It should be one of: B or BSS.")
+  if(length(formula) > 2) stop("This test is not designed for varying correlation coefficient models.")
+
 
   ig <- interpret.gam(formula)
   mf <- match.call(expand.dots = FALSE)
@@ -15,7 +18,7 @@ LM.bpm <- function(formula, data = list(), weights = NULL, subset = NULL,
   fake.formula <- paste(ig[[1]]$response, "~", paste(pred.n, collapse = " + ")) 
   environment(fake.formula) <- environment(ig$fake.formula)
   mf$formula <- fake.formula  
-  mf$Model <- mf$hess <- mf$infl.fac <- NULL  
+  mf$Model <- mf$hess <- NULL  
   mf$drop.unused.levels <- TRUE 
   if(Model=="BSS") mf$na.action <- na.pass
   mf[[1]] <- as.name("model.frame")
@@ -39,7 +42,7 @@ LM.bpm <- function(formula, data = list(), weights = NULL, subset = NULL,
   }
   
 
-  gam1 <- eval(substitute(gam(formula.eq1, binomial(link="probit"), gamma=infl.fac, weights=weights, 
+  gam1 <- eval(substitute(gam(formula.eq1, binomial(link="probit"), weights=weights, 
                               data=data),list(weights=weights))) 
 
   X1 <- model.matrix(gam1)
@@ -47,11 +50,12 @@ LM.bpm <- function(formula, data = list(), weights = NULL, subset = NULL,
   l.sp1 <- length(gam1$sp)
   y1 <- gam1$y
   n <- length(y1) 
+  if(l.sp1 != 0) sp1 <- gam1$sp else sp1 <- NULL 
 
 
   if(Model=="B"){
   
-  gam2  <- eval(substitute(gam(formula.eq2, binomial(link="probit"), gamma=infl.fac, weights=weights, 
+  gam2  <- eval(substitute(gam(formula.eq2, binomial(link="probit"), weights=weights, 
                            data=data),list(weights=weights))) 
   X2 <- model.matrix(gam2)
   X2.d2 <- dim(X2)[2]
@@ -72,14 +76,14 @@ LM.bpm <- function(formula, data = list(), weights = NULL, subset = NULL,
   if(Model=="BSS"){
 
   inde <- y1 > 0
-  gam2 <- eval(substitute(gam(formula.eq2, binomial(link="probit"), gamma=infl.fac, weights=weights, 
+  gam2 <- eval(substitute(gam(formula.eq2, binomial(link="probit"), weights=weights, 
                               data=data, subset=inde),list(weights=weights,inde=inde)))                              
   X2.d2 <- length(coef(gam2))
   X2 <- matrix(0,length(inde),X2.d2,dimnames = list(c(1:length(inde)),c(names(coef(gam2)))) )
   X2[inde, ] <- model.matrix(gam2) 
   y2 <- rep(0,length(inde)); y2[inde] <- gam2$y
   l.sp2 <- length(gam2$sp)
-
+  
   cy1 <- (1-y1)
   y1.y2 <- y1*y2
   y1.cy2 <- y1*(1-y2)
@@ -88,20 +92,14 @@ LM.bpm <- function(formula, data = list(), weights = NULL, subset = NULL,
 
   }
 
-
+  if(l.sp2 != 0) sp2 <- gam2$sp else sp2 <- NULL 
   gp1 <- gam1$nsdf
   gp2 <- gam2$nsdf   
   
   
   
-  if(l.sp1!=0 && l.sp2!=0) sp <- c(gam1$sp,gam2$sp)  # this bit can be improved
-  if(l.sp1==0 && l.sp2!=0) sp <- c(gam2$sp)
-  if(l.sp1!=0 && l.sp2==0) sp <- c(gam1$sp)
-
-
-
-
-  if( (l.sp1!=0 || l.sp2!=0) ) qu.mag <- S.m(gam1, gam2, gam3 = NULL, l.sp1, l.sp2, l.sp3 = 0) 
+  if( l.sp1!=0 || l.sp2!=0){ sp <- c(sp1, sp2)
+  qu.mag <- S.m(gam1, gam2, gam3 = NULL, gam4 = NULL, gam5 = NULL, gam6 = NULL, l.sp1, l.sp2, l.sp3 = 0, l.sp4 = 0, l.sp5 = 0, l.sp6 = 0) }
 
 
   respvec <- list(y1 = y1,
@@ -113,25 +111,23 @@ LM.bpm <- function(formula, data = list(), weights = NULL, subset = NULL,
                   cy1 = cy1)
   
   VC <- list(X1 = X1, 
-             X2 = X2, 
+             X2 = X2, X3 = NULL,
              X1.d2 = X1.d2, 
              X2.d2 = X2.d2,
              gp1 = gp1, 
              gp2 = gp2,
              l.sp1 = l.sp1, 
-             l.sp2 = l.sp2,
-             infl.fac = infl.fac,
+             l.sp2 = l.sp2, l.sp3 = 0,
              weights = weights,
              hess = hess,
              Model = Model,
              end = end, fp = fp,
-             BivD = BivD, nC = 1, extra.regI = FALSE)
+             BivD = BivD, nC = 1, extra.regI = FALSE, margins = c("probit","probit"))
 
 
 params <- c(coef(gam1),coef(gam2),0)
 
-resf <- func.opt(params, sp.xi1 = NULL, sp.xi2 = NULL, PL, eqPL = NULL, valPL = NULL, 
-                 fitPL = NULL, respvec, VC, sp, qu.mag)
+resf <- func.opt(params, respvec, VC, sp, qu.mag)
 
 G   <- resf$gradient
 var <- resf$hessian
@@ -141,6 +137,9 @@ if(min(var.eig$values) < sqrt(.Machine$double.eps)) var.eig$values[which(var.eig
 var <- var.eig$vec%*%tcrossprod(diag(1/var.eig$val),var.eig$vec)  
 
 ev <- as.numeric(t(G)%*%var%*%G)
+
+rm(data, var, G, var.eig, params, resf, VC, respvec, qu.mag, X1, X2, gam1, gam2 )
+
 return(pchisq(ev,1,lower.tail=FALSE))
 
 }
